@@ -24,10 +24,9 @@
 #endif
 
 #include <glib/gstdio.h>
-#include <gnome.h>
 #include <pwd.h>
 #include <gio/gio.h>
-#include <libgnomeui/gnome-thumbnail.h>
+#include <libgnomeui/gnome-desktop-thumbnail.h>
 #include <gconf/gconf-client.h>
 #include <glade/glade.h>
 #include <pwd.h>
@@ -42,6 +41,8 @@
 #define MAX_HEIGHT 150
 #define MAX_WIDTH  150
 
+#define EMAIL_SLOTS 4
+
 typedef struct {
 	EContact 	*contact;
 	EBook    	*book;
@@ -50,10 +51,12 @@ typedef struct {
 
 	GdkScreen    	*screen;
 	GtkIconTheme 	*theme;
-	GnomeThumbnailFactory *thumbs;
+	GnomeDesktopThumbnailFactory *thumbs;
 
 	EContactAddress *addr1;
 	EContactAddress *addr2;
+	gchar           *email[EMAIL_SLOTS];
+	const gchar     *email_types[EMAIL_SLOTS];
 
 	gboolean      	 have_image;
 	gboolean      	 image_changed;
@@ -82,15 +85,15 @@ enum {
 	ADDRESS_COUNTRY
 };
 
-#define EMAIL_HOME              0
-#define EMAIL_WORK              1
+#define EMAIL_WORK              0
+#define EMAIL_HOME              1
 #define ADDRESS_HOME		21
 #define ADDRESS_WORK		27
 
 struct WidToCid ids[] = {
 
-	{ "email-work-e",      E_CONTACT_EMAIL_1             },	/* 00 */
-	{ "email-home-e",      E_CONTACT_EMAIL_2             }, /* 01 */
+	{ "email-work-e",      0                             }, /* 00 */
+	{ "email-home-e",      1                             }, /* 01 */
 
 	{ "phone-home-e",      E_CONTACT_PHONE_HOME          }, /* 02 */
 	{ "phone-mobile-e",    E_CONTACT_PHONE_MOBILE        }, /* 03 */
@@ -134,6 +137,7 @@ struct WidToCid ids[] = {
 
 #define ATTRIBUTE_HOME "HOME"
 #define ATTRIBUTE_WORK "WORK"
+#define ATTRIBUTE_OTHER "OTHER"
 
 static void about_me_set_address_field (EContactAddress *, guint, gchar *);
 
@@ -167,23 +171,54 @@ about_me_destroy (GnomeAboutMe *me)
 	if (me->dialog)
 		g_object_unref (me->dialog);
 
+	g_free (me->email[0]);
+	g_free (me->email[1]);
+	g_free (me->email[2]);
+	g_free (me->email[3]);
+
 	g_free (me->person);
 	g_free (me->login);
 	g_free (me->username);
 	g_free (me);
 }
 
+static void
+about_me_update_email (GnomeAboutMe *me)
+{
+	GList *attrs = NULL;
+	gint i;
+
+	for (i = 0; i < EMAIL_SLOTS; ++i) {
+		if (me->email[i] != NULL) {
+			EVCardAttribute *attr;
+			const gchar *type = me->email_types[i];
+
+			attr = e_vcard_attribute_new (NULL, EVC_EMAIL);
+
+			e_vcard_attribute_add_param_with_value (attr,
+								e_vcard_attribute_param_new (EVC_TYPE),
+								type ? type : ATTRIBUTE_OTHER);
+
+			e_vcard_attribute_add_value (attr, me->email[i]);
+			attrs = g_list_append (attrs, attr);
+		}
+	}
+
+	e_contact_set_attributes (me->contact, E_CONTACT_EMAIL, attrs);
+
+	g_list_foreach (attrs, (GFunc) e_vcard_attribute_free, NULL);
+	g_list_free (attrs);
+}
+
 static void
 about_me_commit (GnomeAboutMe *me)
 {
 	EContactName *name;
-	GError *error;
 
 	char *strings[4], **stringptr;
 	char *fileas;
 
 	name = NULL;
-	error = NULL;
 
 	if (me->create_self) {
 		if (me->username == NULL)
@@ -208,12 +243,14 @@ about_me_commit (GnomeAboutMe *me)
 		g_free (fileas);
 	}
 
+	about_me_update_email (me);
+
 	if (me->create_self) {
-		e_book_add_contact (me->book, me->contact, &error);
-		e_book_set_self (me->book, me->contact, &error);
+		e_book_add_contact (me->book, me->contact, NULL);
+		e_book_set_self (me->book, me->contact, NULL);
 	} else {
-		if (e_book_commit_contact (me->book, me->contact, &error) == FALSE)
-			g_print ("There was an undetermined error\n");
+		if (!e_book_commit_contact (me->book, me->contact, NULL))
+			g_warning ("Could not save contact information");
 	}
 
 	me->create_self = FALSE;
@@ -230,7 +267,7 @@ about_me_commit_from_timeout (GnomeAboutMe *me)
 static gboolean
 about_me_focus_out (GtkWidget *widget, GdkEventFocus *event, GnomeAboutMe *me)
 {
-	gchar *str = NULL;
+	gchar *str;
 	const gchar *wid;
 	gint i;
 
@@ -240,13 +277,9 @@ about_me_focus_out (GtkWidget *widget, GdkEventFocus *event, GnomeAboutMe *me)
 		return FALSE;
 
 	for (i = 0; ids[i].wid != NULL; i++)
-		if (g_ascii_strcasecmp (ids[i].wid, wid) == 0)
+		if (strcmp (ids[i].wid, wid) == 0)
 			break;
 
-	if (ids[i].cid == 0) {
-		return FALSE;
-	}
-
 	if (GTK_IS_ENTRY (widget)) {
 		str = gtk_editable_get_chars (GTK_EDITABLE (widget), 0, -1);
 	} else if (GTK_IS_TEXT_VIEW (widget)) {
@@ -260,11 +293,18 @@ about_me_focus_out (GtkWidget *widget, GdkEventFocus *event, GnomeAboutMe *me)
 		gtk_text_iter_forward_to_end (&iter_end);
 		str = gtk_text_iter_get_text (&iter_start, &iter_end);
 	} else {
-		str = g_strdup ("");
+		return FALSE;
 	}
 
+	if (i == EMAIL_HOME || i == EMAIL_WORK) {
+
+		g_free (me->email[ids[i].cid]);
+		if (str[0] == '\0')
+			me->email[ids[i].cid] = NULL;
+		else
+			me->email[ids[i].cid] = g_strdup (str);
 	/* FIXME: i'm getting an empty address field in evolution */
-	if (i >= ADDRESS_HOME && i < ADDRESS_WORK) {
+	} else if (i >= ADDRESS_HOME && i < ADDRESS_WORK) {
 		about_me_set_address_field (me->addr1, ids[i].cid, str);
 		e_contact_set (me->contact, E_CONTACT_ADDRESS_HOME, me->addr1);
 	} else if (i >= ADDRESS_WORK) {
@@ -292,7 +332,7 @@ get_user_login (void)
 	struct passwd pwd, *err;
 
 	int i;
-#if __sun
+#if defined(__sun) && !defined(_POSIX_PTHREAD_SEMANTICS)
 	i = getpwuid_r (getuid (), &pwd, buf, sizeof (buf));
 	return (i != 0) ? g_strdup (pwd.pw_name) : NULL;
 #else
@@ -346,64 +386,77 @@ about_me_set_address_field (EContactAddress *addr, guint cid, gchar *str)
 {
 	switch (cid) {
 		case ADDRESS_STREET:
-			if (addr->street)
-				g_free (addr->street);
+			g_free (addr->street);
 			addr->street = g_strdup (str);
 			break;
 		case ADDRESS_POBOX:
-			if (addr->po)
-				g_free (addr->po);
+			g_free (addr->po);
 			addr->po = g_strdup (str);
 			break;
 		case ADDRESS_LOCALITY:
-			if (addr->locality)
-				g_free (addr->locality);
+			g_free (addr->locality);
 			addr->locality = g_strdup (str);
 			break;
 		case ADDRESS_CODE:
-			if (addr->code)
-				g_free (addr->code);
+			g_free (addr->code);
 			addr->code = g_strdup (str);
 			break;
 		case ADDRESS_REGION:
-			if (addr->region)
-				g_free (addr->region);
+			g_free (addr->region);
 			addr->region = g_strdup (str);
 			break;
 		case ADDRESS_COUNTRY:
-			if (addr->country)
-				g_free (addr->country);
+			g_free (addr->country);
 			addr->country = g_strdup (str);
 			break;
 	}
 }
 
 static void
-about_me_setup_email (EContact *contact)
+about_me_setup_email (GnomeAboutMe *me)
 {
-	GList *attrs, *a;
+	GList *attrs, *la;
 	gboolean has_home = FALSE, has_work = FALSE;
-	guint cid;
+	guint i;
 
-	attrs = e_contact_get_attributes (contact, E_CONTACT_EMAIL);
+	attrs = e_contact_get_attributes (me->contact, E_CONTACT_EMAIL);
 
-	for (a = attrs, cid = E_CONTACT_EMAIL_1; a; a = a->next, ++cid) {
-		EVCardAttribute *att = (EVCardAttribute *) a->data;
+	for (la = attrs, i = 0; la; la = la->next, ++i) {
+		EVCardAttribute *a = la->data;
 
-		if (!has_home &&
-		    e_vcard_attribute_has_type (att, ATTRIBUTE_HOME)) {
-			has_home = TRUE;
-			ids[EMAIL_HOME].cid = cid;
-		} else if (!has_work &&
-			   e_vcard_attribute_has_type (att, ATTRIBUTE_WORK)) {
-			has_work = TRUE;
-			ids[EMAIL_WORK].cid = cid;
+		me->email[i] = e_vcard_attribute_get_value (a);
+		if (e_vcard_attribute_has_type (a, ATTRIBUTE_HOME)) {
+			me->email_types[i] = ATTRIBUTE_HOME;
+			if (!has_home) {
+				ids[EMAIL_HOME].cid = i;
+				has_home = TRUE;
+			}
+		} else if (e_vcard_attribute_has_type (a, ATTRIBUTE_WORK)) {
+			me->email_types[i] = ATTRIBUTE_WORK;
+			if (!has_work) {
+				ids[EMAIL_WORK].cid = i;
+				has_work = TRUE;
+			}
+		} else {
+			me->email_types[i] = ATTRIBUTE_OTHER;
 		}
-		e_vcard_attribute_free (att);
+
+		e_vcard_attribute_free (a);
 	}
 
 	g_list_free (attrs);
+
+	if (ids[EMAIL_HOME].cid == ids[EMAIL_WORK].cid) {
+		if (has_home)
+			ids[EMAIL_WORK].cid = 1;
+		else
+			ids[EMAIL_HOME].cid = 0;
+	}
+
+	me->email_types[ids[EMAIL_WORK].cid] = ATTRIBUTE_WORK;
+	me->email_types[ids[EMAIL_HOME].cid] = ATTRIBUTE_HOME;
 }
+
 /**
  * about_me_load_string_field:
  *
@@ -428,7 +481,9 @@ about_me_load_string_field (GnomeAboutMe *me, const gchar *wid, guint cid, guint
 		return;
 	}
 
-	if (aid >= ADDRESS_HOME && aid < ADDRESS_WORK) {
+	if (aid == EMAIL_HOME || aid == EMAIL_WORK) {
+		str = e_contact_get_const (me->contact, E_CONTACT_EMAIL_1 + cid);
+	} else if (aid >= ADDRESS_HOME && aid < ADDRESS_WORK) {
 		str = about_me_get_address_field (me->addr1, cid);
 	} else if (aid >= ADDRESS_WORK) {
 		str = about_me_get_address_field (me->addr2, cid);
@@ -461,7 +516,8 @@ about_me_load_photo (GnomeAboutMe *me, EContact *contact)
 
 	widget = WID ("image-chooser");
 
-	e_image_chooser_set_from_file (E_IMAGE_CHOOSER (widget), me->person);
+	if (me->person)
+		e_image_chooser_set_from_file (E_IMAGE_CHOOSER (widget), me->person);
 
 	photo = e_contact_get (contact, E_CONTACT_PHOTO);
 
@@ -564,10 +620,11 @@ about_me_update_photo (GnomeAboutMe *me)
 		/* Update the image in the card */
 		e_contact_set (me->contact, E_CONTACT_PHOTO, NULL);
 
-		/* Update GDM configuration */
-		gnome_config_set_string ("/gdmphotosetup/last/picture", "");
-		gnome_config_set_string ("/gdm/face/picture", "");
-		gnome_config_sync ();
+		file = g_build_filename (g_get_home_dir (), ".face", NULL);
+
+		g_unlink (file);
+
+		g_free (file);
 	}
 
 	about_me_commit (me);
@@ -623,7 +680,7 @@ about_me_update_preview (GtkFileChooser *chooser,
 		GFileInfo *file_info;
 
 		if (!me->thumbs)
-			me->thumbs = gnome_thumbnail_factory_new (GNOME_THUMBNAIL_SIZE_NORMAL);
+			me->thumbs = gnome_desktop_thumbnail_factory_new (GNOME_DESKTOP_THUMBNAIL_SIZE_NORMAL);
 
 		file = g_file_new_for_uri (uri);
 		file_info = g_file_query_info (file,
@@ -641,7 +698,7 @@ about_me_update_preview (GtkFileChooser *chooser,
 
 				mime_type = g_content_type_get_mime_type (content_type);
 
-				pixbuf = gnome_thumbnail_factory_generate_thumbnail (me->thumbs,
+				pixbuf = gnome_desktop_thumbnail_factory_generate_thumbnail (me->thumbs,
 										     uri,
 										     mime_type);
 				g_free (mime_type);
@@ -827,7 +884,7 @@ about_me_setup_dialog (void)
 
 	/* Connect the close button signal */
 	main_dialog = WID ("about-me-dialog");
-	g_signal_connect (G_OBJECT (main_dialog), "response",
+	g_signal_connect (main_dialog, "response",
 			  G_CALLBACK (about_me_button_clicked_cb), me);
 
 	gtk_window_set_resizable (GTK_WINDOW (main_dialog), FALSE);
@@ -838,10 +895,10 @@ about_me_setup_dialog (void)
 	me->theme = gtk_icon_theme_get_for_screen (me->screen);
 
 	icon = gtk_icon_theme_lookup_icon (me->theme, "stock_person", 80, 0);
-
-	me->person = g_strdup (gtk_icon_info_get_filename (icon));
-
-	gtk_icon_info_free (icon);
+	if (icon != NULL) {
+		me->person = g_strdup (gtk_icon_info_get_filename (icon));
+		gtk_icon_info_free (icon);
+	}
 
 	g_signal_connect_object (me->theme, "changed",
 				 G_CALLBACK (about_me_icon_theme_changed),
@@ -877,7 +934,7 @@ about_me_setup_dialog (void)
 			}
 		}
 	} else {
-		about_me_setup_email (me->contact);
+		about_me_setup_email (me);
 	}
 
 	/************************************************/
@@ -894,7 +951,7 @@ about_me_setup_dialog (void)
 
 	/************************************************/
 
-	if (tok[0] == NULL || strlen (tok[0]) == 0)
+	if (tok[0] == NULL || *tok[0] == '\0')
 		me->username = NULL;
 	else
 		me->username = g_strdup (tok[0]);
@@ -903,7 +960,7 @@ about_me_setup_dialog (void)
 	about_me_load_photo (me, me->contact);
 
 	widget = WID ("fullname");
-	if (tok[0] == NULL || strlen (tok[0]) == 0) {
+	if (tok[0] == NULL || *tok[0] == '\0') {
 		str = g_strdup_printf ("<b><span size=\"xx-large\">%s</span></b>", me->login);
 	} else {
 		str = g_strdup_printf ("<b><span size=\"xx-large\">%s</span></b>", tok[0]);
@@ -915,7 +972,7 @@ about_me_setup_dialog (void)
 	widget = WID ("login");
 	gtk_label_set_text (GTK_LABEL (widget), me->login);
 
-	if (tok[0] == NULL || strlen (tok[0]) == 0) {
+	if (tok[0] == NULL || *tok[0] == '\0') {
 		str = g_strdup_printf (_("About %s"), me->login);
 	} else {
 		str = g_strdup_printf (_("About %s"), tok[0]);
@@ -925,15 +982,15 @@ about_me_setup_dialog (void)
 	g_strfreev (tok);
 
 	widget = WID ("password");
-	g_signal_connect (G_OBJECT (widget), "clicked",
+	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (about_me_passwd_clicked_cb), me);
 
 	widget = WID ("button-image");
-	g_signal_connect (G_OBJECT (widget), "clicked",
+	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (about_me_image_clicked_cb), me);
 
 	widget = WID ("image-chooser");
-	g_signal_connect (G_OBJECT (widget), "changed",
+	g_signal_connect (widget, "changed",
 			  G_CALLBACK (about_me_image_changed_cb), me);
 
 	/* Address tab: set up the focus chains */
@@ -967,17 +1024,9 @@ about_me_setup_dialog (void)
 int
 main (int argc, char **argv)
 {
-	GnomeProgram *program;
 	int rc = 0;
 
-	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
-	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-	textdomain (GETTEXT_PACKAGE);
-
-	program = gnome_program_init ("gnome-about-me", VERSION,
-			LIBGNOMEUI_MODULE, argc, argv,
-			GNOME_PARAM_APP_DATADIR, GNOMECC_DATA_DIR,
-			NULL);
+	capplet_init (NULL, &argc, &argv);
 
 	rc = about_me_setup_dialog ();
 
@@ -985,7 +1034,5 @@ main (int argc, char **argv)
 		gtk_main ();
 	}
 
-	g_object_unref (program);
-
 	return rc;
 }
